#ifndef _SPECTRUM_H_
#define _SPECTRUM_H_

#include "image.h"
#include <math.h>

typedef struct _spectrum {
  double *real;
  double *imag;
  int ncols,nrows;
  int *tbrow;
} Spectrum;

int       FFT(int dir, long nn, double *x, double *y);
Spectrum *FFT2D(Image *img);
Image    *InvFFT2D(Spectrum *spec);
Spectrum *CreateSpectrum(int ncols, int nrows);
void      DestroySpectrum(Spectrum **spec);
Image    *ViewMagnitude(Spectrum *spec);
Image    *ViewPhase(Spectrum *spec);
Spectrum *SpectrumMagnitude(Spectrum *spec);
Spectrum *SpectrumPhase(Spectrum *spec);

/******************** Funes da questo 3 ****************************/

/* Gera o espectro real com ncols x nrows pixels de um filtro de
   Butterworth do tipo passa-baixas freqncias, onde Dl indica a
   freqncia de corte e n o fator de decaimento.  */

Spectrum *LowPassFilter(int ncols, int nrows, float Dl, float n);

/* Gera o espectro real com ncols x nrows pixels de um filtro de
   Butterworth do tipo passa-altas freqncias, onde Dh indica a
   freqncia de corte e n o fator de decaimento. */

Spectrum *HighPassFilter(int ncols, int nrows, float Dh, float n);

/* Gera o espectro real com ncols x nrows pixels de um filtro de
   Butterworth do tipo passa-faixa, onde Dl e Dh indicam as
   freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandPassFilter(int ncols, int nrows, float Dl, float nl, float Dh, 
float nh);

/* Gera o espectro real com ncols x nrows pixels de um filtro de
   Butterworth do tipo rejeita-faixa, onde Dl e Dh indicam as
   freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandRejectFilter(int ncols, int nrows, float Dl, float nl, float 
Dh, float nh);

/**************** Funes da questo 4 
****************************************/

Spectrum *MoorePenrose(int n, int N); /* Cria a inversa de
                                         Moore-Penrose para
                                         transformar um espectro com
                                         (N+1)x(N+1) pixels em uma
                                         mscara com (n+1)x(n+1)
                                         coeficientes */


Image *ComputeMask(Spectrum *spc, Spectrum *moore, int n); /* Cria uma
                                             mscara espacial com
                                             (n+1)x(n+1) coeficientes
                                             a partir do espectro spc
                                             de um filtro com
                                             (N+1)x(N+1)
                                             pixels. Observe que os
                                             coeficientes calculados
                                             sero em ponto flutuante,
                                             mas devem ser
                                             multiplicados por uma
                                             constante e arredondados
                                             para valores inteiros
                                             antes de serem gravados
                                             na imagem de
                                             sada. Observe tambm que
                                             o espectro spc no 
                                             centrado. Ele deve ser
                                             deslocado para o centro
                                             antes de aplicar a
                                             inversa de Moore-Penrose
                                             para que a mscara fique
                                             centrada e que toda
                                             operao envolva apenas
                                             funes reais e
                                             simtricas. Da
                                             garantimos que a mscara
                                             ser centrada. Veja como
                                             centralizar o espectro em
                                             ViewMagnitude. */

#endif
